﻿-- ShuckIt.lua
-- $Id: ShuckIt.lua 25 2009-11-22 03:31:47Z LaoTseu $

-- The ShuckIt idea came from the Shucks module from Cork (http://www.wowinterface.com/downloads/info10629) by Tekkub.
-- The basic framework for this addon is based on Tekkub's AddonTemplate (http://github.com/tekkub/addontemplate/tree/master)
-- The code to find the openable items is based on Ammo's work in 
-- CrowBar (http://www.wowace.com/addons/crow-bar/) and is used with his permission.


ShuckIt = ShuckIt or {}
ShuckIt.revision = ShuckIt.revision or {} 
ShuckIt.revision.main	= ("$Revision: 25 $"):match("(%d+)")
ShuckIt.version      	= GetAddOnMetadata("ShuckIt", "Version"):match("([^ ]+)")
ShuckIt.revision.toc  	= GetAddOnMetadata("ShuckIt", "Version"):match("%$Revision:%s(%d+)")

----------------------
--      Locals      --
----------------------

local L = ShuckIt.localized
--local defaults, defaultsPC, db, dbpc = {}, {}
local UPDATEPERIOD, elapsed = 0.5, 0
local need_scan, need_chat_update, busy, delay_scan = nil, nil, nil, nil

local item_to_shuck, icon, shuck_count = nil, nil, 0
local shuck_table = {}
local shuck_bag, shuck_slot = 1, 1
local default_icon = "Interface\\Icons\\INV_Misc_Shell_03"

local secureframe

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("ShuckIt", 
											 {
												type = "data source", 
											  	text = "",
											  	icon = default_icon
})


------------------------------
--      Util Functions      --
------------------------------

local default_channel = nil
local function setDefaultChanelForPrint()
	default_channel = nil
	for i=1, NUM_CHAT_WINDOWS do
		local name = GetChatWindowInfo(i)
		if name and strlower(name) == 'output' then
			default_channel = i
		end
	end
end

local function Print(message, ...) 
	if default_channel then
		_G["ChatFrame"..default_channel]:AddMessage(format("|cff00dbba"..L["ShuckIt"].."|r: "..message, ...));
	else
		SELECTED_CHAT_FRAME:AddMessage(format("|cff00dbba"..L["ShuckIt"].."|r: "..message, ...))
	end
end

local debugf = tekDebug and tekDebug:GetFrame("ShuckIt")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

------------------------------
--      Core Functions     --
------------------------------

CreateFrame("GameTooltip", "ShuckItTooltip")
ShuckItTooltip:SetOwner(WorldFrame, "ANCHOR_NONE")

ShuckItTooltip:AddFontStrings(
    ShuckItTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
    ShuckItTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
)

local cache = {}
local do_not_cache = {}
--SunkIt_cache = cache
local function IsOpenable(bag, slot, itemLink)
	--local _,_,itemID = string.find(itemLink, "^[|]c%x+[|]H[^:]+[:]([^:]+)[:].*[|]h%[.+%]")
	--local found,_,itemID = string.find(itemLink, "item[:](%d+)[:]")

	--assert(found,"No itemID: " .. gsub(itemLink, "\124", "\124\124"))
	
	if not bag or not slot then return false end
	if not itemLink then itemLink = GetContainerItemLink(bag, slot) end
	
	if cache[itemLink] ~= nil then return cache[itemLink] end
	
	ShuckItTooltip:ClearLines()
	if not ShuckItTooltip:IsOwned(WorldFrame) then ShuckItTooltip:SetOwner(WorldFrame, "ANCHOR_NONE") end
 	ShuckItTooltip:SetBagItem(bag,slot)
	
	for i=1,ShuckItTooltip:NumLines() do
	   local mytext = getglobal("ShuckItTooltipTextLeft" .. i)
	   local text = mytext:GetText()

		if (text == ITEM_OPENABLE) and not do_not_cache[itemLink] then
			if not do_not_cache[itemLink] then 
				cache[itemLink] = true
			end
			return true
		elseif text == LOCKED then
			-- Do not cache the result for locked items, they may become unlocked
			do_not_cache[itemLink] = true
			return false
		end
	end
	
	cache[itemLink] = false
	return false

end

local cache_clam = {}
local function IsAClam(bag, slot, itemLink)
	if not bag or not slot then return false end
	if not itemLink then itemLink = GetContainerItemLink(bag, slot) end

	if cache_clam[itemLink] ~= nil then return cache_clam[itemLink] end
	
	ShuckItTooltip:ClearLines()
	if not ShuckItTooltip:IsOwned(WorldFrame) then ShuckItTooltip:SetOwner(WorldFrame, "ANCHOR_NONE") end
 	ShuckItTooltip:SetBagItem(bag,slot)

	for i=1,ShuckItTooltip:NumLines() do
	   local mytext = getglobal("ShuckItTooltipTextLeft" .. i)
	   local text = mytext:GetText()

		if (text == L["Use: Open the clam!"]) then
			cache_clam[itemLink] = true
			return true
		end
	end
	
	cache_clam[itemLink] = false
	return false

end

local function ScanBags()

	item_to_shuck, shuck_bag, shuck_slot = nil, nil, nil
	shuck_count = 0
	for k in pairs(shuck_table) do shuck_table[k] = nil end

	local newIcon = nil
	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do

			local itemLink = GetContainerItemLink(bag, slot)
			if itemLink and (IsOpenable(bag, slot, itemLink) or IsAClam(bag, slot, itemLink)) then
				local _,_,itemID = string.find(itemLink, "item[:](%d+)[:]")
				shuck_table[itemID] = (shuck_table[itemID] or 0) + tonumber(select(2,GetContainerItemInfo(bag, slot)) or 1)
				if not item_to_shuck then
					item_to_shuck = GetItemInfo(itemLink)
					shuck_bag, shuck_slot = bag, slot
					newIcon = GetContainerItemInfo(bag, slot)
				end
			elseif itemLink and not GetItemInfo(itemLink) then
				-- This is a workaround for a intermitend situation when opening
				-- multiple items of the same kind in a row sometimes result in
				-- the item information being screewed up.
				-- If this happen, we delay de scan until the information is realiable
				delay_scan = true
				return
			end

		end
	end
	
	for k in pairs (shuck_table) do
		shuck_count = shuck_count + shuck_table[k]
	end
	icon = newIcon or default_icon
	need_scan = nil
end

local function myOnClick(self, button, down) 
	if not InCombatLockdown() and not IsStealthed() and not busy and IsOpenable(shuck_bag, shuck_slot) then
		UseContainerItem(shuck_bag, shuck_slot)
		return true
	elseif not InCombatLockdown() and not IsStealthed() and not busy and IsAClam(shuck_bag, shuck_slot) then
		Print(L["Cannot open Clams by clicking on the LDB display. Use |cffffd700/click ShuckItFrame|r instead."])
	else
		return false
	end
end

-----------------------------
--      Event Handler      --
-----------------------------

local f = CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
f:RegisterEvent("ADDON_LOADED")


function f:ADDON_LOADED(event, addon)
	if addon ~= "ShuckIt" then return end

	--AddonTemplateDB, AddonTemplateDBPC = setmetatable(AddonTemplateDB or {}, {__index = defaults}), setmetatable(AddonTemplateDBPC or {}, {__index = defaultsPC})
	--db, dbpc = AddonTemplateDB, AddonTemplateDBPC

	-- Do anything you need to do after addon has loaded

	--LibStub("tekKonfig-AboutPanel").new("ShuckIt", "ShuckIt") -- Remove first arg if no parent config panel

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil

	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end


function f:PLAYER_LOGIN()
	self:RegisterEvent("PLAYER_LOGOUT")

	-- Register events
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("UPDATE_CHAT_WINDOWS")

	-- Start the OnUpdate 
	self:SetScript("OnUpdate", function(self, elap)
		elapsed = elapsed + elap
		
		if delay_scan then 
			-- Workaround for when good item information is not available and we
			-- need to wait a bit
			elapsed = UPDATEPERIOD/2
			need_scan = true
			delay_scan = nil
			return
		end
		
		if not (need_scan or need_chat_update) or busy or elapsed < UPDATEPERIOD or InCombatLockdown() then return end

		elapsed = 0
		
		-- Print window if needed
		if need_chat_update then
			setDefaultChanelForPrint()
			need_chat_update = nil
		end

		-- Find object to Shuck
		ScanBags()
		
		-- Adjust the display
		if shuck_count > 0 then
			dataobj.text = string.format(L["%s - %d"], item_to_shuck, shuck_count)
		else
			dataobj.text = ""
		end
		dataobj.icon = icon
	end)
	
	-- Set the OnClick function for the data broker
	dataobj.OnClick = myOnClick
	
	-- Find the revision number
	if ShuckIt.revision then
		ShuckIt.rev = 0
		for _, rev in pairs(ShuckIt.revision) do
			local revision = tonumber(rev)
			if revision and revision > ShuckIt.rev then ShuckIt.rev = revision end
		end
	else
		assert(false,"No ShuckIt.revision")
	end

	-- Find the default channel
	setDefaultChanelForPrint()
	
	need_scan = true

	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end


function f:PLAYER_LOGOUT()
	--for i,v in pairs(defaults) do if db[i] == v then db[i] = nil end end
	--for i,v in pairs(defaultsPC) do if dbpc[i] == v then dbpc[i] = nil end end

	-- Do anything you need to do as the player logs out
end

function f:BAG_UPDATE(bagID)
	elapsed = 0 -- Let's wait a bit
	need_scan = true
end

function f:UNIT_SPELLCAST_SENT(event, unit, spell, rank, target)
	busy = true
end

function f:UNIT_SPELLCAST_FAILED(event, unit, spell, rank)
	busy = nil
end

function f:UNIT_SPELLCAST_STOP(event, unit, spell, rank)
	busy = nil
end

function f:UNIT_SPELLCAST_INTERRUPTED(event, unit, spell, rank)
	busy = nil
end

local lock_picking_spell = GetSpellInfo(1804)
function f:UNIT_SPELLCAST_SUCCEEDED(event, unit, spell, rank)
	busy = nil
	if unit == "player" and spell == lock_picking_spell then
		-- A something just got unlocked, it can be opened
		elapsed = -UPDATEPERIOD -- Let's wait a bit
		need_scan = true
	end
end

function f:UPDATE_CHAT_WINDOWS()
	need_chat_update = true
end


-----------------------------
--      Slash Handler      --
-----------------------------

SLASH_ADDONTEMPLATE1 = "/shuckit"
SlashCmdList.ADDONTEMPLATE = function(msg)
	Print(L["Use |cffffd700/click ShuckItFrame|r to open the next item"])
end

-----------------------------
--        LDB Stuff        --
-----------------------------

-- Sort function
local function sort_by_item_name (a,b)
	local name_a = GetItemInfo(a)
	local name_b = GetItemInfo(b)
	return name_a < name_b
end


local index = {}
function dataobj:OnTooltipShow()
	self:AddLine(format("%s v%s (r%d)", L["ShuckIt"], ShuckIt.version, ShuckIt.rev))
	if shuck_count == 0 then return end
	
	self:AddLine(' ')
	self:AddLine(format(L["%d items to open"],shuck_count))
	
	-- clean index
	for key in pairs(index) do index[key] = nil end
	
	-- Sort by item name
	for key in pairs(shuck_table) do tinsert(index,key) end
	sort(index, sort_by_item_name)

	for _, link in ipairs (index) do
		local count = shuck_table[link]
		local name, _, _, _, _, _, _, _, _, texture = GetItemInfo(link)
		self:AddLine(format(L["%s x %d"],name,count),1,1,1)
		self:AddTexture(texture)
	end
end

----------------------------
--      Secure frame      --
----------------------------

secureframe = CreateFrame("Button", "ShuckItFrame", UIParent, "SecureActionButtonTemplate")

secureframe:SetScript("PreClick", function(self)
	if InCombatLockdown() or IsStealthed() or busy or 
			(not IsOpenable(shuck_bag, shuck_slot) and not IsAClam(shuck_bag, shuck_slot)) then 
		Print(L["Nothing to do or in combat"])
		return 
	end
	self:SetAttribute( "type1", "macro" )
	self:SetAttribute( "macrotext1", "/use "..shuck_bag.." "..shuck_slot)
end)


secureframe:SetScript("PostClick", function(self)
	if InCombatLockdown() then return end
	self:SetAttribute( "type1", ATTRIBUTE_NOOP )
	self:SetAttribute( "macrotext1", nil)
end)
